﻿using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;
using Infragistics.Documents.Excel;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace XamGridCellMergeExport
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = DataUtil.Products;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            xamGrid1.GroupBySettings.GroupByOperation = GroupByOperation.GroupByRows;
            xamGrid1.GroupBySettings.GroupByColumns.Add(xamGrid1.Columns["CategoryID"] as Column);
            Export(xamGrid1);
            xamGrid1.GroupBySettings.GroupByOperation = GroupByOperation.MergeCells;           
        }

        public void Export(object obj) 
        {
            var dialog = new SaveFileDialog { Filter = "Excel files|*.xlsx", DefaultExt = "xlsx" };

            bool? ret = dialog.ShowDialog();
            if (ret != true) return;

            try
            {
                Stream exportStream = dialog.OpenFile();

                //launch the excel export
                Workbook dataWorkbook = this.SaveExcel(obj, "sheetName");
                dataWorkbook.Save(exportStream);
                exportStream.Close();

                MessageBox.Show(
                    "The file " + dialog.SafeFileName + " was successfully generated.");

            }
            catch (IOException)
            {
                MessageBox.Show(
                        "Impossible to acces to file.");
            }
        }

        #region Variables

        private const int InitialColumnsCount = 0;
        private int _currentMergedRow;
        private int _currentMergedColumn;
        private int _countGrouped;
        private int _countNotGrouped;
        private int _outlineLevel = 1;
        private int _outlineLevelNextGroup = 1;
        private int _childrenCount;
        private Worksheet _sheet;
        private IWorksheetCellFormat _iwshcfValide;

        #endregion

        /// <summary>
        /// Method to iterate each row of the grid and save it in the excel
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        private Workbook SaveExcel(object obj, string name)
        {
            //get the XamGrid
            var grille = (XamGrid)obj;
            //creation od the workbook
            var workbook = new Workbook(WorkbookFormat.Excel2007);

            //sheet name
            _sheet = workbook.Worksheets.Add(name + " - " + DateTime.Now.Day + " " + DateTime.Now.Month + " " + DateTime.Now.Year);
            
            // Samples to define the style of cells
            //Sample 1
            _iwshcfValide = workbook.CreateNewWorksheetCellFormat();
            _iwshcfValide.Alignment = HorizontalCellAlignment.Left;

            _currentMergedRow = 0;
            _currentMergedColumn = 0;

            //get the groups, if there's 
            
            var groupByQuery = from row in grille.Rows
                               where row.RowType == RowType.GroupByRow
                               select row;

            //get the simplerows
            var simpleQuery = from row in grille.Rows
                              where row.RowType == RowType.DataRow
                              select row;

            foreach (var query in groupByQuery)
            {
                _childrenCount = 0;
                _countGrouped = 0;
                _countNotGrouped = 0;
                _childrenCount = GetQueryElementItemsCount(query as GroupByRow);
                _outlineLevel = 1;
                _outlineLevelNextGroup = 2;


                //this 'for' create the +/- in front of each group to collapse them
                for (int i = 0; i < _childrenCount; i++)
                    _sheet.Rows[_currentMergedRow + i + 1].OutlineLevel = _outlineLevel;

                _currentMergedColumn = 0;

                //treatment of the rows of the group
                IterateGroupByRow(query as GroupByRow, _outlineLevelNextGroup);
            }

            foreach (var query in simpleQuery)
            {
                //the fisrt row do not scroll with others when ther's no groups
                _sheet.DisplayOptions.PanesAreFrozen = true;
                _sheet.DisplayOptions.FrozenPaneSettings.FrozenRows = 1;

                int currentColumn = 0;
                //creat the header when no groups
                foreach (Column column in grille.Columns)
                {
                    if (column.Visibility == Visibility.Visible)
                    {
                        this.SetCellValue(_sheet.Rows[0].Cells[currentColumn], column.HeaderText);
                        currentColumn++;
                    }
                }
                IterateRow(query);
            }

            return workbook;
        }

        /// <summary>
        /// Permet de connaitre le nombre d'element d'un groupe (ligne + entête)
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        private int GetQueryElementItemsCount(GroupByRow query)
        {
            RowCollection children = query.Rows;
            foreach (Row child in children)
            {
                if (child.RowType == RowType.GroupByRow)
                {
                    _countGrouped++;
                    GetQueryElementItemsCount(child as GroupByRow);
                }
                else
                    _countNotGrouped++;
            }
            return _countGrouped + _countNotGrouped;
        }

        /// <summary>
        /// Method to check each row of a group
        /// </summary>
        /// <param name="groupByRow"></param>
        /// <param name="outlinelevel">le niveau du groupe pour les regroupements dans excel</param>
        private void IterateGroupByRow(GroupByRow groupByRow, int outlinelevel)
        {
            //merge to get an indentation
            WorksheetMergedCellsRegion mergedRegion = _sheet.MergedCellsRegions.Add(_currentMergedRow, _currentMergedColumn, _currentMergedRow, _currentMergedColumn + InitialColumnsCount);
            _sheet.Columns[_currentMergedColumn].Width = 6000;
            if (groupByRow.GroupByData.Value == null)
                mergedRegion.Value = "";
            else
            {
                try
                {
                    DateTime dt = DateTime.ParseExact(groupByRow.GroupByData.Value.ToString(), "dd/MM/yyyy HH:mm:ss", new CultureInfo("en-US"));
                    mergedRegion.Value = dt.Date.ToShortDateString();
                }
                catch (FormatException)
                {
                    mergedRegion.Value = groupByRow.GroupByData.Value;
                }
            }

            _currentMergedRow++;
            _currentMergedColumn++;

            if (groupByRow.HasChildren)
            {
                RowCollection groupByRowChildren = groupByRow.Rows;

                //check for each row if it's a group or not
                foreach (Row child in groupByRowChildren)
                {
                    _countGrouped = 0;
                    _countNotGrouped = 0;
                    if (child.RowType == RowType.GroupByRow)
                    {
                        //boucle permettant d'afficher le +/- sur le cote de la feuille Excel pour chaque premier groupe
                        int temp = this.GetQueryElementItemsCount(child as GroupByRow);

                        for (int i = 0; i < temp; i++)
                            _sheet.Rows[_currentMergedRow + i + 1].OutlineLevel = outlinelevel;

                        IterateGroupByRow(child as GroupByRow, outlinelevel + 1);

                        _currentMergedColumn--;
                    }
                    else
                        IterateRow(child);
                }
            }
        }

        /// <summary>
        /// Save a line
        /// </summary>
        /// <param name="row"></param>
        private void IterateRow(Row row)
        {
            int columnIndex = _currentMergedColumn;

            foreach (Cell cell in row.Cells)
            {
                //get the values switch the type of the cell
                if (cell.Column is ComboBoxColumn)
                {
                    string path = ((ComboBoxColumn)(cell.Column)).DisplayMemberPath;
                    string value = (string)cell.Value.GetType().GetProperty(path).GetValue(cell.Value, null);
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = value;
                }
                else if (cell.Column is CheckBoxColumn)
                {
                    var header = cell.Column.HeaderText;
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = Boolean.Parse(cell.Value.ToString()) ? header != null ? header.ToLower() : "Checked" : header != null ? "Not" + header.ToLower() : "Unchecked";
                }
                else if (cell.Column is DateColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((DateColumn)(cell.Column)).SelectedDateFormat;
                }
                else if (cell.Column is HyperlinkColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((HyperlinkColumn)(cell.Column)).Content;
                }
                else if (cell.Column is ImageColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = "Image not supported yet";
                }
                else if (cell.Column is TemplateColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((TemplateColumn)(cell.Column)).ItemTemplate.ToString();
                }
                else if (cell.Column is UnboundColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((UnboundColumn)(cell.Column)).ItemTemplate.ToString();
                }
                else
                    _sheet.Rows[_currentMergedRow].Cells[columnIndex].Value = cell.Value;

                //Sample to apply a particular style to a row
                _sheet.Rows[_currentMergedRow].Cells[columnIndex].CellFormat.SetFormatting(_iwshcfValide);

                //affect a width to be sur to see the entire row (could be improve)
                _sheet.Columns[columnIndex].Width = 6000;
                columnIndex++;
            }
            _currentMergedRow++;
        }

        /// <summary>
        /// Method to apply a particular style to a cell and affect a value
        /// </summary>
        /// <param name="cell"></param>
        /// <param name="value"></param>
        private void SetCellValue(WorksheetCell cell, object value)
        {
            cell.Value = value;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;
            cell.CellFormat.Font.Bold = ExcelDefaultableBoolean.True;
        }

        private void xamGrid1_Loaded(object sender, RoutedEventArgs e)
        {
            //Column col = xamGrid1.Columns["CategoryID"];
            xamGrid1.GroupBySettings.GroupByColumns.Add(xamGrid1.Columns["CategoryID"] as Column);
        }
    }

    public class DataUtil
    {
        #region private members

        #region private fields

        private static ObservableCollection<Category> categories;
        private static ObservableCollection<Product> products;
        private static ObservableCollection<Category> categoriesAndProducts;

        #endregion

        #region private methods

        private static ObservableCollection<Category> CreateCategories()
        {
            return new ObservableCollection<Category>
            {
                new Category 
                {
                    CategoryID = 1, 
                    CategoryName = "Beverages", 
                    Description = "Soft drinks, coffees, teas, beers, and ales"
                },

                new Category 
                {
                    CategoryID = 2, 
                    CategoryName = "Condiments", 
                    Description = "Sweet and savory sauces, relishes, spreads, and seasonings"
                },
                
                new Category
                {
                    CategoryID = 3,
                    CategoryName = "Confections",
                    Description = "Desserts, candies, and sweet breads",
                },

                new Category
                {
                    CategoryID = 4,
                    CategoryName = "Dairy Product",
                    Description = "Cheeses",
                },

                new Category
                {
                    CategoryID = 5,
                    CategoryName = "Grains/Cereals",
                    Description = "Breads, crackers, pasta, and cereal",
                },

                new Category
                {
                    CategoryID = 6,
                    CategoryName = "Meat/Poultry",
                    Description = "Prepared meats",
                },

                new Category
                {
                    CategoryID = 7,
                    CategoryName = "Produce",
                    Description = "Dried fruit and bean curd",
                },

                new Category
                {
                    CategoryID = 8,
                    CategoryName = "Seafood",
                    Description = "Seaweed and fish",
                }
            };
        }

        private static ObservableCollection<Product> CreateProducts()
        {
            return new ObservableCollection<Product>
            {
                new Product
                {
                    ProductID = 1,
                    ProductName = "Chai",
                    CategoryID = 1,
                    QuantityPerUnit = "10 boxes x 20 bags",
                    UnitPrice = 18.0000m,
                    UnitsInStock = 39,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 2,
                    ProductName = "Chang",
                    CategoryID = 1,
                    QuantityPerUnit = "24 - 12 oz bottles",
                    UnitPrice = 19.0000m,
                    UnitsInStock = 17,
                    UnitsOnOrder = 40,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 24,
                    ProductName = "Guarana Fantastica",
                    CategoryID = 1,
                    QuantityPerUnit = "12 - 355 ml cans",
                    UnitPrice = 4.5000m,
                    UnitsInStock = 20,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 77,
                    ProductName = "Original Frankfurter grune So?e",
                    CategoryID = 2,
                    QuantityPerUnit = "12 boxes",
                    UnitPrice = 13.0000m,
                    UnitsInStock = 32,
                    UnitsOnOrder = 0,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 61,
                    ProductName = "Sirop d'erable",
                    CategoryID = 2,
                    QuantityPerUnit = "24 - 500 ml bottles",
                    UnitPrice = 28.5000m,
                    UnitsInStock = 113,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 63,
                    ProductName = "Vegie-spread",
                    CategoryID = 2,
                    QuantityPerUnit = "15 - 625 g jars",
                    UnitPrice = 43.9000m,
                    UnitsInStock = 24,
                    UnitsOnOrder = 0,
                    ReorderLevel = 5,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 16,
                    ProductName = "Pavlova",
                    CategoryID = 3,
                    QuantityPerUnit = "32 - 500 g boxes",
                    UnitPrice = 17.4500m,
                    UnitsInStock = 29,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 19,
                    ProductName = "Teatime Chocolate Biscuits",
                    CategoryID = 3,
                    QuantityPerUnit = "10 boxes x 12 pieces",
                    UnitPrice = 9.2000m,
                    UnitsInStock = 25,
                    UnitsOnOrder = 0,
                    ReorderLevel = 5,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 20,
                    ProductName = "Sir Rodney's Marmalade",
                    CategoryID = 3,
                    QuantityPerUnit = "30 gift boxes",
                    UnitPrice = 81.0000m,
                    UnitsInStock = 40,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 69,
                    ProductName = "Gudbrandsdalsost",
                    CategoryID = 4,
                    QuantityPerUnit = "10 kg pkg.",
                    UnitPrice = 36.0000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 71,
                    ProductName = "Flotemysost",
                    CategoryID = 4,
                    QuantityPerUnit = "10 - 500 g pkgs.",
                    UnitPrice = 21.5000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 72,
                    ProductName = "Mozzarella di Giovanni",
                    CategoryID = 4,
                    QuantityPerUnit = "24 - 200 g pkgs.",
                    UnitPrice = 34.8000m,
                    UnitsInStock = 14,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },
                new Product
                {
                    ProductID = 22,
                    ProductName = "Gustaf's Knackebrod",
                    CategoryID = 5,
                    QuantityPerUnit = "24 - 500 g pkgs.",
                    UnitPrice = 21.0000m,
                    UnitsInStock = 104,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 23,
                    ProductName = "Tunnbrod",
                    CategoryID = 5,
                    QuantityPerUnit = "12 - 250 g pkgs.",
                    UnitPrice = 9.0000m,
                    UnitsInStock = 61,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 53,
                    ProductName = "Perth Pasties",
                    CategoryID = 6,
                    QuantityPerUnit = "48 pieces",
                    UnitPrice = 32.8000m,
                    UnitsInStock = 0,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 54,
                    ProductName = "Tourtiere",
                    CategoryID = 6,
                    QuantityPerUnit = "16 pies",
                    UnitPrice = 7.4500m,
                    UnitsInStock = 21,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 55,
                    ProductName = "Pate chinois",
                    CategoryID = 6,
                    QuantityPerUnit = "24 boxes x 2 pies",
                    UnitPrice = 24.0000m,
                    UnitsInStock = 115,
                    UnitsOnOrder = 0,
                    ReorderLevel = 20,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 28,
                    ProductName = "Rossle Sauerkraut",
                    CategoryID = 7,
                    QuantityPerUnit = "25 - 825 g cans",
                    UnitPrice = 45.6000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 7,
                    ProductName = "Uncle Bob's Organic Dried Pears",
                    CategoryID = 7,
                    QuantityPerUnit = "12 - 1 lb pkgs.",
                    UnitPrice = 30.0000m,
                    UnitsInStock = 15,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 14,
                    ProductName = "Tofu",
                    CategoryID = 7,
                    QuantityPerUnit = "40 - 100 g pkgs.",
                    UnitPrice = 23.2500m,
                    UnitsInStock = 35,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 45,
                    ProductName = "Rogede sild",
                    CategoryID = 8,
                    QuantityPerUnit = "1k pkg.",
                    UnitPrice = 9.5000m,
                    UnitsInStock = 5,
                    UnitsOnOrder = 70,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 46,
                    ProductName = "Spegesild",
                    CategoryID = 8,
                    QuantityPerUnit = "4 - 450 g glasses",
                    UnitPrice = 12.0000m,
                    UnitsInStock = 95,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 58,
                    ProductName = "Escargots de Bourgogne",
                    CategoryID = 8,
                    QuantityPerUnit = "24 pieces",
                    UnitPrice = 13.2500m,
                    UnitsInStock = 62,
                    UnitsOnOrder = 0,
                    ReorderLevel = 20,
                    Discontinued = false
                }
            };
        }

        private static ObservableCollection<Category> CreateCategoriesAndProducts()
        {
            ObservableCollection<Category> retVal = CreateCategories();

            ObservableCollection<Product> tempProducts = CreateProducts();

            Category parentCategory = null;

            foreach (Product p in tempProducts)
            {
                if (parentCategory == null || parentCategory.CategoryID != p.CategoryID)
                {
                    parentCategory = retVal.First<Category>(category => category.CategoryID == p.CategoryID);
                }

                parentCategory.Products.Add(p);
            }

            return retVal;
        }

        #endregion

        #endregion

        #region public members

        #region public properties
        public static ObservableCollection<Category> Categories
        {
            get
            {
                if (categories == null)
                    categories = CreateCategories();

                return categories;
            }
        }

        public static ObservableCollection<Product> Products
        {
            get
            {
                if (products == null)
                    products = CreateProducts();

                return products;
            }
            set 
            {
                products = value;
            }
        }

        public static ObservableCollection<Category> CategoriesAndProducts
        {
            get
            {
                if (categoriesAndProducts == null)
                    categoriesAndProducts = CreateCategoriesAndProducts();

                return categoriesAndProducts;
            }
        }
        #endregion

        #endregion
    }

    public class Category : INotifyPropertyChanged
    {
        #region private fields

        private int categoryID;
        private string categoryName;
        private string description;
        private ObservableCollection<Product> products;

        #endregion

        #region public properties
        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                    OnPropertyChanged("CategoryID");
                }
            }
        }

        public string CategoryName
        {
            get
            {
                return this.categoryName;
            }
            set
            {
                if (this.categoryName != value)
                {
                    this.categoryName = value;
                    OnPropertyChanged("CategoryName");
                }
            }
        }

        public string Description
        {
            get
            {
                return this.description;
            }
            set
            {
                if (this.description != value)
                {
                    this.description = value;
                    OnPropertyChanged("Description");
                }
            }
        }

        public ObservableCollection<Product> Products
        {
            get
            {
                if (products == null)
                {
                    products = new ObservableCollection<Product>();
                }

                return this.products;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }

    public class Product : INotifyPropertyChanged
    {

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        #region private members

        private int productID;
        private string productName;
        private int categoryID;
        private string quantityPerUnit;
        private decimal unitPrice;
        private int unitsInStock;
        private int unitsOnOrder;
        private int reorderLevel;
        private bool discontinued;

        #endregion

        #region public properties

        public int ProductID
        {
            get
            {
                return this.productID;
            }
            set
            {
                if (this.productID != value)
                {
                    this.productID = value;
                    OnPropertyChanged("ProductID");
                }
            }
        }

        public string ProductName
        {
            get
            {
                return this.productName;
            }
            set
            {
                if (this.productName != value)
                {
                    this.productName = value;
                    OnPropertyChanged("ProductName");
                }
            }
        }

        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                    OnPropertyChanged("CategoryID");
                }
            }
        }

        public string QuantityPerUnit
        {
            get
            {
                return this.quantityPerUnit;
            }
            set
            {
                if (this.quantityPerUnit != value)
                {
                    this.quantityPerUnit = value;
                    OnPropertyChanged("QuantityPerUnit");
                }
            }
        }

        public decimal UnitPrice
        {
            get
            {
                return this.unitPrice;
            }
            set
            {
                if (value < 0)
                    throw new Exception("Negative numbers are not allowed.");

                this.unitPrice = value;
                OnPropertyChanged("UnitPrice");
            }
        }

        public int UnitsInStock
        {
            get
            {
                return this.unitsInStock;
            }
            set
            {
                if (value < 0)
                    throw new Exception("Negative numbers are not allowed.");

                this.unitsInStock = value;
                OnPropertyChanged("UnitsInStock");
            }
        }

        public int UnitsOnOrder
        {
            get
            {
                return this.unitsOnOrder;
            }
            set
            {
                if (this.unitsOnOrder != value)
                {
                    this.unitsOnOrder = value;
                    OnPropertyChanged("UnitsOnOrder");
                }
            }
        }

        public int ReorderLevel
        {
            get
            {
                return this.reorderLevel;
            }
            set
            {
                if (this.reorderLevel != value)
                {
                    this.reorderLevel = value;
                    OnPropertyChanged("ReorderLevel");
                }
            }
        }

        public bool Discontinued
        {
            get
            {
                return this.discontinued;
            }
            set
            {
                if (this.discontinued != value)
                {
                    this.discontinued = value;
                    OnPropertyChanged("Discontinued");
                }
            }
        }

        #endregion
    }

}
